import re
from urllib.parse import urlparse, parse_qs

import requests
import time
from urlextract import URLExtract

from ..api import g_tk
from ..exceptions import ClientkeyLoginException

__all__ = [
    "login_by_clientkey"
]

login_datas = [
    {
        "proxy_url": "https://qzs.qq.com/qzone/v6/portal/proxy.html",
        "daid": "5",
        "hide_title_bar": "1",
        "low_login": "0",
        "qlogin_auto_login": "1",
        "no_verifyimg": "1",
        "link_target": "blank",
        "appid": "549000912",
        "style": "22",
        "target": "self",
        "s_url": "https://qzs.qq.com/qzone/v5/loginsucc.html?para=izone",
        "pt_qr_app": "手机QQ空间",
        "pt_qr_link": "https://z.qzone.com/download.html",
        "self_regurl": "https://qzs.qq.com/qzone/v6/reg/index.html",
        "pt_qr_help_link": "https://z.qzone.com/download.html",
        "pt_no_auth": "0"
    },
    {
        "pt_disable_pwd": "1",
        "appid": "715030901",
        "hide_close_icon": "1",
        "daid": "73",
        "pt_no_auth": "1",
        "s_url": "https://qun.qq.com/"
    },
    # {"u1":"https://wx.mail.qq.com/list/readtemplate?name=login_page.html","s_url":None},
    {
        "target": "self",
        "appid": "522005705",
        "daid": "4",
        "s_url": "https://wx.mail.qq.com/list/readtemplate?name=login_jump.html",
        "style": "25",
        "low_login": "1",
        "proxy_url": "https://mail.qq.com/proxy.html",
        "need_qr": "0",
        "hide_border": "1",
        "border_radius": "0",
        "self_regurl": "https://reg.mail.qq.com",
        "app_id": "11005?t=regist",
        "pt_feedback_link": "http://support.qq.com/discuss/350_1.shtml",
        "css": "https://res.mail.qq.com/zh_CN/htmledition/style/ptlogin_input_for_xmail.css",
        "enable_qlogin": "0"
    },
    {
        "appid": "8000201",
        "style": "20",
        "s_url": "https://vip.qq.com/loginsuccess.html",
        "maskOpacity": "60",
        "daid": "18",
        "target": "self"
    },
    {
        "appid": "527020901",
        "daid": "372",
        "low_login": "0",
        "qlogin_auto_login": "1",
        "s_url": "https://www.weiyun.com/web/callback/common_qq_login_ok.html?login_succ",
        "style": "20",
        "hide_title": "1",
        "target": "self",
        "link_target": "blank",
        "hide_close_icon": "1",
        "pt_no_auth": "1",
        'u1': "https://www.weiyun.com/?adtag=ntqqmainpanel"
    },
    {
        "style": "40",
        "appid": "1600001573",
        "s_url": "https://accounts.qq.com/homepage#/",
        "daid": "761",
        "hide_close_icon": "0"
    },
    {
        "appid": "10000101",
        "s_url": "https://qqshow.qq.com/manage/myCreation",
        "hide_close_icon": "1"
    },
    {
        "s_url": "https://huifu.qq.com/recovery/index.html?frag=1",
        "style": "20",
        "appid": "715021417",
        "daid": "768",
        "proxy_url": "https://huifu.qq.com/proxy.html"
    },
    {"u1": "https://docs.qq.com/desktop/?tdsourcetag=s_ntpcqq_panel_app", "s_url": None},
    {
        "daid": "377",
        "style": "11",
        "appid": "716027613",
        "target": "self",
        "pt_disable_pwd": "1",
        "s_url": "https://connect.qq.com/login_success.html",
        "t": str(time.time())
    },
    {
        "appid": "501038301",
        "target": "self",
        "s_url": "https://im.qq.com/loginSuccess"
    }
]

def login_by_clientkey(uin:str, clientkey:str, which:int=0, force_clientkey_url:bool=False) -> str:
    """
    通过clientkey登录
    :param uin: 账号
    :param clientkey: clientkey
    :param which: 登录方式，0为QQ空间，1为QQ群，2为QQ邮箱，3为QQ会员，
                  4为微云，5为QQ账号中心，6为QQ秀，7为QQ恢复中心，
                  8为腾讯文档，9为QQ互联，10为腾讯IM
    :param force_clientkey_url: 是否强制使用clientkey登录网址，默认False(https://ssl.ptlogin2.qq.com/jump?...),如没有将报KeyError(仅支持which=0,1,4)
    :return: 登录网址
    """
    if force_clientkey_url and which not in [0, 4]:
        raise ClientkeyLoginException(f"Not support force_clientkey_url")
    login_datas[0]['u1'] = f"https://user.qzone.qq.com/{uin}/infocenter"
    session = requests.session()
    login_data = login_datas[which]
    if force_clientkey_url or not login_data['s_url']:
        return f"https://ssl.ptlogin2.qq.com/jump?ptlang={1033 if len(clientkey) == 96 else 2052}&clientuin={uin}&clientkey={clientkey}&u1={login_data['u1']}&keyindex={19 if len(clientkey) == 96 else 9}"
    if which == 2 and len(clientkey) == 96:
        login_html = session.get(
            "https://xui.ptlogin2.qq.com/cgi-bin/xlogin?appid=716027609&daid=383&style=33&login_text=%E7%99%BB%E5%BD%95&hide_title_bar=1&hide_border=1&target=self&s_url=https%3A%2F%2Fgraph.qq.com%2Foauth2.0%2Flogin_jump&pt_3rd_aid=102013353&pt_feedback_link=https%3A%2F%2Fsupport.qq.com%2Fproducts%2F77942%3FcustomInfo%3D.appid102013353&theme=10&verify_theme=")
        x_cookies = login_html.cookies.get_dict()
        pt_local_token = x_cookies.get("pt_local_token")
        session.cookies.update({"clientuin": uin, "clientkey": clientkey})
        jump_req = session.get("https://ssl.ptlogin2.qq.com/jump", params={
            "clientuin": uin,
            "keyindex": "19",
            "pt_aid": "716027609",
            "daid": "383",
            "u1": "https://graph.qq.com/oauth2.0/login_jump",
            "pt_local_tk": pt_local_token,
            "pt_3rd_aid": "102013353",
            "ptopt": "1",
            "style": "40",
            "has_onekey": "1"
        })
        sigx_jump_url = re.findall("'(.*?)'", jump_req.text)[1]
        session.get(sigx_jump_url)
        auth_data = {
            "response_type": "code",
            "client_id": "102013353",
            "redirect_uri": "https://wx.mail.qq.com/list/readtemplate?name=login_jump.html",
            "scene": "1",
            "login_type": "qq",
            "scope": "get_user_info,get_app_friends",
            "state": "",
            "switch": "",
            "from_ptlogin": "1",
            "src": "1",
            "update_auth": "1",
            "openapi": "1010",
            "g_tk": g_tk(session.cookies.get("p_skey")),
            "auth_time": int(time.time() * 1000),
        }
        auth_req = session.post("https://graph.qq.com/oauth2.0/authorize", data=auth_data, json=auth_data,
                                allow_redirects=False)
        redirect_url = urlparse(auth_req.headers['Location'])
        code = parse_qs(redirect_url.query)['code'][0]
        login_url = f"https://wx.mail.qq.com/list/readtemplate?name=login_jump.html&scene=1&login_type=qq&from=qq_panel&code={code}"
        return login_url
    if login_data['s_url']:
        login_htm = session.get(
            "https://xui.ptlogin2.qq.com/cgi-bin/xlogin", params=login_data)
        q_cookies = requests.utils.dict_from_cookiejar(login_htm.cookies)
        pt_local_token = q_cookies.get("pt_local_token")
        headers = {"Referer": "https://xui.ptlogin2.qq.com/",
                   "Host": "ssl.ptlogin2.qq.com",
                   "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36 Edg/123.0.0.0"}
        params = {
            "u1": login_data['s_url'],
            "clientuin": uin,
            "pt_aid": login_data['appid'],
            "keyindex": "19",
            "pt_local_tk": pt_local_token,
            "pt_3rd_aid": "0",
            "ptopt": "1",
            "style": "40"
        }
        if login_data.get("daid"): params['daid'] = login_data.get("daid")
        cookies = {
            "clientkey": clientkey,
            "clientuin": str(uin),
            "pt_local_token": pt_local_token
        }
        login_res = session.get("https://ssl.ptlogin2.qq.com/jump", params=params, cookies=cookies, headers=headers)
        extractor = URLExtract()
        login_url = extractor.find_urls(login_res.text)[0]
        return login_url



import requests
from ..api import *
from html import unescape

__all__ = ["QunManager"]

class QunManager:

    __slots__ = ['uin', 'skey', 'p_skey', 'bkn', 'cookies', 'headers']

    def __init__(self, uin, skey:str, p_skey:str):
        self.uin = str(uin)
        self.skey = skey
        self.p_skey = p_skey
        self.bkn = bkn(skey)
        self.cookies = {
            "p_skey": self.p_skey,
            "uin": self.uin,
            "skey": self.skey,
            "p_uin": self.uin,
            "ptui_loginuin":str(uin)
        }
        self.headers = {
            "Referer": "https://web.qun.qq.com/mannounce/index.html?_wv=1031&_bid=148",
            "User-Agent": "Mozilla/5.0 (Linux; Android 10; MI 8 Build/QKQ1.190910.002; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/109.0.5414.86 MQQBrowser/6.2 TBS/046715 Mobile Safari/537.36 V1_AND_SQ_8.9.78_4548_YYB_D QQ/8.9.78.12275 NetType/WIFI WebP/0.3.0 AppId/537175315 Pixel/1080 StatusBarHeight/82 SimpleUISwitch/0 QQTheme/2099 StudyMode/0 CurrentMode/0 CurrentFontScale/1.0 GlobalDensityScale/0.9818182 AllowLandscape/false InMagicWin/0",
            "Host": "web.qun.qq.com",
            "Origin": "https://web.qun.qq.com"
        }

    def add_qun_notice(self, content:str, target_qid, pinned:int=0, confirm_required:int=0, to_json:bool=False):
        """
        发送群公告，需要提供详细参数
        :param content: 公告内容
        :param target_qid: 目标群号
        :param pinned: 是否置顶,默认0(不需要)
        :param confirm_required: 是否需要确定收到,默认0(不需要)
        :param to_json: 是否转为json，默认False
        :return: api所返回的内容
        """
        data = {
            "qid": target_qid,
            "bkn": self.bkn,
            "text": content,
            "pinned": pinned,
            "type": 1,
            "settings": {"is_show_edit_card": 1, "tip_window_type": 0, "confirm_required": confirm_required}
        }
        r1 = requests.post(f"https://web.qun.qq.com/cgi-bin/announce/add_qun_notice?bkn={self.bkn}", params=data,headers=self.headers, cookies=self.cookies)
        r1.encoding = "utf-8"
        return r1.json() if to_json else r1.text

    def get_notices_list(self, target_qid, replace_newline:str='\n') -> list:
        """
        获取群公告
        :param target_qid: 目标群号
        :param replace_newline: 指定替换换行字符的内容,默认\n
        :return: 以json为格式的群公告列表
        """
        data = {
            "qid": target_qid,
            "bkn": self.bkn,
            "n":"10",
            "s":"-1",
            "ni":"1",
            "i":"1",
            "ft":"23"
        }
        notices = []
        while True:
            res = requests.post(f"https://web.qun.qq.com/cgi-bin/announce/list_announce?bkn={self.bkn}", params=data,json=data,headers=self.headers, cookies=self.cookies)
            res.encoding = 'utf-8'
            res = res.json()
            if "feeds" in res or "inst" in res:
                data['s'] = str(int(data['s'])-10)
                data['ni'] = 'undefined'
                try:
                    for j in res['inst']:
                        _ = {
                            'uin':j['u'],
                            'fid':j['fid'],
                            'content':unescape(j['msg']['text']).replace("\n",replace_newline)
                        }
                        if _ in notices:
                            break
                        notices.append(_)
                except:
                    pass
                try:
                    for j in res['feeds']:
                        _ = {
                            'uin':j['u'],
                            'fid':j['fid'],
                            'content':unescape(j['msg']['text']).replace("\n",replace_newline)
                        }
                        if _ in notices:
                            break
                        notices.append(_)
                except:
                    pass
            else:
                break
        return notices

    def delete_notice(self, target_qid, fid:str, to_json:bool=False) -> dict:
        """
        删除群公告，需要fid，可通过get_notices_list获取
        :param target_qid: 目标群号
        :param fid: 对应公告的fid
        :param to_json: 是否转为json，默认False
        :return: api返回的结果
        """
        data = {
            "qid": target_qid,
            "bkn": self.bkn,
            "fid": fid
        }
        r1 = requests.post(f"https://web.qun.qq.com/cgi-bin/announce/del_feed?bkn={bkn}", data=data,params=data,headers=self.headers, cookies=self.cookies)
        r1.encoding = 'utf-8'
        return r1.json() if to_json else r1.text

    def get_qun_members(self, target_qid) -> list:
        """
        获取群成员列表
        :param target_qid: 目标群号
        :return: 群成员列表
        """
        data = {
            "st":"0",
            "start":"0",
            "end":"9",
            "sort":"1",
            "group_id":target_qid,
            "gc":target_qid
        }
        l = []
        start = 0
        end = 9
        while True:
            res = requests.post(f"https://qun.qq.com/cgi-bin/qun_mgr/search_group_members?bkn={self.bkn}&ts=1702901784527",cookies=self.cookies,data=data,headers=self.headers).json()
            res.encoding = 'utf-8'
            try:
                for i in res['mems']:
                    _ = {
                        'uin': i['uin'],
                        'nickname': i['nick'],
                        'join_time': i['join_time'],
                        'last_speak_time': i['last_speak_time']
                    }
                    if _ in l:
                        break
                    l.append(_)
                start += 10
                end += 10
                data['start'],data['end'] = str(start),str(end)
            except:
                break
        return l


from warnings import warn

import requests
import random
import base64
import time
from urlextract import URLExtract
import re
from .api import *
from . import tool

__all__ = ['QrLogin','ClientkeyLogin']

class QrLogin:

    __slots__ = ['s_url','qrsig','ptqrtoken','g_tk','login_url','daid','appid']

    def __init__(self, url="qzone.qq.com", custom_data=None):
        """
        初始化对象,设置对应登录业务的s_url(u1)
        :param url: 登录url,留空默认使用QQ空间
        :param custom_data: 是否自定义登录数据,如需自定义需按照格式输入,否则从自带字典选取
        """
        app_list = {
            "qzone.qq.com":{"s_url":"https://qzs.qq.com/qzone/v5/loginsucc.html?para=izone","daid":"5","appid":"549000912"},
            "qun.qq.com":{"s_url":"https://qun.qq.com","daid":"73","appid":"715030901"},
            "vip.qq.com":{"s_url":"https://vip.qq.com/loginsuccess.html","daid":"18","appid":"8000201"}
        }
        if custom_data:data = custom_data
        else: data = app_list.get(url)
        self.s_url = data.get("s_url")
        self.daid = data.get("daid")
        self.appid = data.get("appid")
        self.qrsig = None
        self.ptqrtoken = None
        self.login_url = None

    def getQrcode(self, base64_encode=False) -> dict:
        """
        根据s_url获取二维码,返回以base64编码的qrcode图片(如果base64_encode为True,否则返回图片content),qrsig与ptqrtoken
        :return: qrsig,ptqrtoken
        """
        url = f"https://ssl.ptlogin2.qq.com/ptqrshow?appid={self.appid}&t={random.random()}&daid={self.daid}&pt_3rd_aid=0&u1={self.s_url}"
        qr_req = requests.get(url)
        qr_req.encoding = 'utf-8'
        qr_cookies = qr_req.cookies.get_dict()
        qrcode_content = qr_req.content
        if base64_encode:
            qrcode_content = base64.b64encode(qrcode_content)
        qrsig = qr_cookies.get("qrsig")
        ptqrtoken = ptqrToken(qrsig)
        self.qrsig = qrsig
        self.ptqrtoken = ptqrtoken
        if not (qrsig and ptqrtoken):
            return {"code":-1,"msg":"获取qrsig或ptqrtoken失败!"}
        return {"code":0,"msg":"获取成功!","img":qrcode_content,"qrsig":qrsig,"ptqrtoken":ptqrtoken}

    def check_scanning_status(self) -> dict:
        """
        检测二维码扫描状态,并返回QQ号和登录地址
        :return: 状态以及QQ号和登录网址
        """
        if not (self.qrsig and self.ptqrtoken):
            return {"code":-1,"msg":"请先获取二维码!"}
        l = requests.get(
            f"https://ssl.ptlogin2.qq.com/ptqrlogin?u1={self.s_url}&ptqrtoken={self.ptqrtoken}&ptredirect=0&h=1&t=1&g=1&from_ui=1&ptlang=2052&action={time.time()}&js_ver=23111510&js_type=1&login_sig=&pt_uistyle=40&aid={self.appid}&daid={self.daid}&&o1vId=&pt_js_version=v1.48.1",
            cookies={"qrsig": self.qrsig})
        l.encoding = 'utf-8'
        if '二维码未失效' in l.text:
            return {"code":0,"msg":"二维码未失效"}
        elif '二维码已失效' in l.text:
            return {"code":-1,"msg":"二维码已失效"}
        elif '二维码认证中' in l.text:
            return {"code":0,"msg":"二维码认证中"}
        elif '拒绝' in l.text:
            return {"code":-1,"msg":"用户已拒绝"}
        elif '成功' in l.text:
            uin = requests.utils.dict_from_cookiejar(l.cookies).get('uin')
            extractor = URLExtract()
            url = extractor.find_urls(l.text)[0]
            self.login_url = url
            return {"code":1,"msg":"登录成功","uin":uin,"login_url":url}
        else:
            return {"code":-2,"msg":"未知状态,请查阅return_msg!","return_msg":l.text}

    def getCookies(self) -> dict:
        """
        访问登录网址获取Cookies
        :return: Cookies
        """
        try:
            login_req = requests.get(self.login_url,allow_redirects=False)
            login_req.encoding = 'utf-8'
            targetCookies = login_req.cookies.get_dict()
            return {"code":0,"msg":"获取成功!","cookies":targetCookies}
        except:
            return {"code":-1,"msg":"获取失败!"}

class ClientkeyLogin:

    __slots__ = ['accounts','pt_local_token','pt_login_sig','session']

    def __init__(self):
        self.accounts = []
        self.pt_local_token = None
        self.pt_login_sig = None
        self.session = requests.session()

    def getClientkey(self, port=4301) -> dict:
        """
        获取Clientkey,需提供端口,默认为4301
        :return: QQ号,名称和clientkey
        """
        try:
            session = requests.session()
            login_html = session.get(
                "https://xui.ptlogin2.qq.com/cgi-bin/xlogin?s_url=https://qzs.qq.com/qzone/v5/loginsucc.html?para=izone")
            q_cookies = login_html.cookies.get_dict()
            pt_local_token = q_cookies.get("pt_local_token")
            params = {
                "callback": "ptui_getuins_CB",
                "r": "0.8987470931280881",
                "pt_local_tk": pt_local_token
            }
            headers = {
                "Referer": "https://xui.ptlogin2.qq.com/",
                "Host": f"localhost.ptlogin2.qq.com:{port}",
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36 Edg/123.0.0.0"
            }
            get_uins_req = session.get(f"https://localhost.ptlogin2.qq.com:{port}/pt_get_uins", params=params,
                                       cookies=q_cookies,
                                       headers=headers, timeout=3)
            uins = re.findall('"uin":(\d+)', get_uins_req.text)
            nickname = re.findall('"nickname":"(.*?)"', get_uins_req.text)
            result = []
            i = 0
            for uin in uins:
                params = {
                    "clientuin": uin,
                    "r": "0.9059695467741018",
                    "pt_local_tk": pt_local_token,
                    "callback": "__jp0"
                }
                ck_req = session.get(f"https://localhost.ptlogin2.qq.com:{port}/pt_get_st", params=params,
                                     cookies=q_cookies, headers=headers)
                ck_cookies = ck_req.cookies.get_dict()
                if len(nickname) == len(uins):
                    ck_cookies["nickname"] = nickname[i]
                else:
                    get_nickname_req = requests.get(
                        f'https://users.qzone.qq.com/fcg-bin/cgi_get_portrait.fcg?uins={uins[i]}')
                    get_nickname_req.encoding = 'utf-8'
                    name = re.findall(r'\[.*,.*,.*,.*,.*,.*,"(.*)",.*\]', get_nickname_req.text)[0]
                    ck_cookies['nickname'] = name
                result.append(ck_cookies)
                i += 1
            self.accounts = result
            return {"code":0, "msg":"获取成功!", "data": result}
        except Exception as e:
            return {"code":-1,"msg":repr(e)}

    def getLoginUrl(self, uin:str="0", index:int=None) -> dict:
        """
        本函数已被弃用,请使用tool.login_by_clientkey代替
        通过Clientkey获取登录url,包含ptsigx登录方式与QQ客户端登录方式对应的url,默认使用最后一个获取到的账号
        uin和index参数都是指定账号，但uin提供的是QQ号而index提供的是账号在列表哪个位置(注意有uin就忽略index参数)
        如果需要更详细
        :param uin: 获取哪个账号的登录网址，有此参数则忽略index参数
        :param: index: 获取第几个账号的登录网址
        :return: login_url
        """
        warn("本函数已被弃用,请使用tool.login_by_clientkey代替", DeprecationWarning)
        try:
            if index == None:
                index = self.getUinIndex(uin)
            uin = self.accounts[index].get("clientuin")
            clientkey = self.accounts[index].get("clientkey")
            qzone_url_ptsigx = tool.login_by_clientkey(uin, clientkey, 0)
            qun_url_ptsigx = tool.login_by_clientkey(uin, clientkey, 1)
            mail_url_clientkey = tool.login_by_clientkey(uin, clientkey, 2)
            qzone_url_cientkey = tool.login_by_clientkey(uin, clientkey, 0, force_clientkey_url=True)
            return {"code":0,"msg":"获取成功!","ptsigx_url":{"qzone":qzone_url_ptsigx, "qun":qun_url_ptsigx},"clientkey_url":{"qzone":qzone_url_cientkey,"mail":mail_url_clientkey}}
        except Exception as e:
            return {"code":-1,"msg":repr(e)}

    def getUinIndex(self, uin: str) -> int:
        """
        获取QQ号所在的索引
        :param uin: QQ号
        :return: 对应的索引,没找到返回-1
        """
        for i in range(len(self.accounts)):
            if self.accounts[i].get("clientuin") == uin:
                return i
        else:
            return -1


import requests
import re

__all__ = [
    "get_avatar_url",
    "get_nickname"
]

def get_avatar_url(uin:str, size:int=100) -> str:
    """
    获取QQ头像URL
    :param uin: QQ号
    :param size: 头像大小，默认为100像素，可选值为40、100、140、640
    :return: 头像URL
    """
    return f"https://q1.qlogo.cn/g?b=qq&nk={uin}&s={size}"

def get_nickname(uin:str) -> tuple:
    """
    获取QQ昵称
    :param uin: QQ号
    :return: 昵称与api返回的原始信息
    """
    get_nickname_req = requests.get(f'https://users.qzone.qq.com/fcg-bin/cgi_get_portrait.fcg?uins={uins[i]}')
    get_nickname_req.encoding = 'utf-8'
    name = re.findall(r'\[.*,.*,.*,.*,.*,.*,"(.*)",.*\]', get_nickname_req.text)[0]
    return name, get_nickname_req.text


import requests
from ..api import *
import json

__all__ = ["QzoneManager"]

class QzoneManager:

    def __init__(self, uin:str, skey:str, p_skey:str):
        self.uin = 'o' + uin if uin[0] != 'o' else uin
        self.skey = skey
        self.p_skey = p_skey
        self.g_tk = g_tk(p_skey)
        self.cookies = {
            "p_skey": p_skey,
            "uin": uin,
            "skey": skey,
            "p_uin": uin
        }
        self.headers = {
            "Referer": f"https://user.qzone.qq.com/",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36 Edg/123.0.0.0",
            "Origin": "https://user.qzone.qq.com"
        }

    def change_name(self, name:str):
        """
        修改昵称
        :param name: 要改的昵称
        :return: api返回的结果
        """
        data = {
    "nickname":name,
    "emoji":"",
    "sex":"1",
    "birthday":"1900-1-1",
    "province":"44",
    "city":"0",
    "country":"1",
    "marriage":"0",
    "bloodtype":"5",
    "hp":"0",
    "hc":"0",
    "hco":"0",
    "career":"",
    "company":"",
    "cp":"0",
    "cc":"0",
    "cb":"",
    "cco":"0",
    "lover":"",
    "islunar":"0",
    "mb":"1",
    "uin":f"{self.uin[1:]}",
    "pageindex":"1",
    "fupdate":"1",
    "qzreferrer":"https://user.qzone.qq.com/proxy/domain/qzs.qq.com/qzone/v6/setting/profile/profile.html?tab=base",
    "g_iframeUser":"1"
}
        r1 = requests.post(
            f"https://h5.qzone.qq.com/proxy/domain/w.qzone.qq.com/cgi-bin/user/cgi_apply_updateuserinfo_new?&g_tk={self.g_tk}",
            data=data, headers=self.headers, params=data, cookies=self.cookies)
        r1.encoding = 'utf-8'
        return r1.text
    def get_friends_list(self) -> list:
        """
        获取好友列表
        :return: 好友列表
        """
        data = {
    "uin":self.uin[1:],
    "do":"1",
    "rd":f"0.928374978349",
    "fupdate":"1",
    "clean":"1",
    "g_tk":self.g_tk
}
        friends = []
        try:
            res = requests.get("https://user.qzone.qq.com/proxy/domain/r.qzone.qq.com/cgi-bin/tfriend/friend_ship_manager.cgi",params=data,cookies=self.cookies,headers=self.headers,json=data,data=data)
            res.encoding = 'utf-8'
            res = res.text[10:-2]
            friend_list = json.loads(res).get("data").get("items_list")
            print(friend_list)
        except Exception as e:
            return [repr(e)]
        for i in friend_list:
            friends.append({
                'name':i['name'],
                'uin':str(i['uin']),
                'img':i['img']
            })
        return friends

    def publish_emotion(self, content:str):
        """
        发表说说
        :param content: 说说的内容
        :return: api返回的结果
        """
        data = {
    "syn_tweet_verson":"1",
    "paramstr":"1",
    "pic_template":"",
    "richtype":"",
    "richval":"",
    "special_url":"",
    "subrichtype":"",
    "who":"1",
    "con":content,
    "feedversion":"1",
    "ver":"1",
    "ugc_right":"1",
    "to_sign":"0",
    "hostuin":self.uin[1:],
    "code_version":"1",
    "format":"fs",
    "qzreferrer":f"https://user.qzone.qq.com/{self.uin[1:]}"
}
        r1 = requests.post(f"https://user.qzone.qq.com/proxy/domain/taotao.qzone.qq.com/cgi-bin/emotion_cgi_publish_v6?&g_tk={self.g_tk}", data=data,headers=self.headers,params=data,cookies=self.cookies).text
        r1.encoding = 'utf-8'
        return r1

    def get_emotions_list(self, page=0, num:int=1, replace_newline:str='\n') -> list:
        """
        获取说说
        :param page: 页数,默认0
        :param num: 获取数量,需为int,默认1
        :param replace_newline: 指定替换换行符的内容
        :return: 说说
        """
        data = {
    "uin":self.uin[1:],
    "inCharset":"utf-8",
    "outCharset":"utf-8",
    "hostUin":self.uin[1:],
    "notice":"0",
    "sort":"0",
    "pos":str(page),
    "num":str(num),
    "cgi_host":"https://user.qzone.qq.com/proxy/domain/taotao.qq.com/cgi-bin/emotion_cgi_msglist_v6",
    "code_version":"1",
    "format":"json",
    "need_private_comment":"1",
    "g_tk":self.g_tk
}
        count = 0
        emotions_list = []
        while True:
            res = requests.get(
                f"https://user.qzone.qq.com/proxy/domain/taotao.qq.com/cgi-bin/emotion_cgi_msglist_v6",
                params=data, headers=self.headers, cookies=self.cookies)
            res.encoding = 'utf-8'
            res = res.json()
            if res.get("msglist"):
                msg_list = res['msglist']
                for msg in msg_list:
                    if count >= num:
                        return emotions_list
                    _ = {
                        "content":msg.get("content").replace("\n",replace_newline),
                        "createtime":msg.get("createTime"),
                        "tid":msg.get("tid"),
                        "img_url": msg['pic'][0]['url1'] if msg.get("pic") else None,
                        "video_url": msg['video'][0]['url3'] if msg.get("video") else None,
                    }
                    if _ in emotions_list:
                        continue
                    emotions_list.append(_)
                    count += 1
                data['pos'] = str(int(data['pos'])+1)
            else:
                break
        return emotions_list

    def delete_emotion(self, tid:str):
        """
        删除说说
        :param tid: 说说的tid
        :return: api返回的结果
        """
        data = {
    "hostuin":self.uin[1:],
    "tid":tid,
    "t1_source":"1",
    "code_version":"1",
    "format":"fs",
    "qzreferrer":f"https://user.qzone.qq.com/{self.uin[1:]}"
}
        return requests.post(f"https://user.qzone.qq.com/proxy/domain/taotao.qzone.qq.com/cgi-bin/emotion_cgi_delete_v6?&g_tk={self.g_tk}",json=data,data=data,cookies=self.cookies,headers=self.headers).text



from setuptools import setup, find_packages
from PyQQSkeyTool.constant import AUTHOR,VERSION,EMAIL

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='PyQQSkeyTool',
    version=VERSION,
    author=AUTHOR,
    author_email=EMAIL,
    description=long_description,
    url='https://github.com/sun589/PyQQSkeyTool',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'requests>=2.25.1',
        'urlextract>=1.6.0'
    ],
    python_requires='>=3.6'
)



"""
本文件为api文件，用于计算各种数据
"""

def ptqrToken(qrsig) -> int:
    """
    计算ptqrtoken,需要qrsig
    :param qrsig: 获取二维码获得的qrsig
    :return: ptqrtoken
    """

    n, i, e = len(qrsig), 0, 0

    while n > i:
        e += (e << 5) + ord(qrsig[i])
        i += 1

    return 2147483647 & e

def bkn(skey):
    """
    计算bkn,需要skey
    :param skey: Cookies中的skey
    :return: bkn
    """

    t,n,o = 5381,0,len(skey)

    while n < o:
        t += (t << 5) + ord(skey[n])
        n += 1

    return t & 2147483647

def g_tk(p_skey):
    """
    计算g_tk,需要p_skey
    :param p_skey: Cookies里的p_skey
    :return: g_tk
    """

    t = 5381
    for i in p_skey:
        t += (t << 5) + ord(i)
    return t & 2147483647